# Aug. 29/12
#
# Code for the homotopic connectivity analyses of 
# Reiss, Huang, Chen, Huo, Tarpey and Mennes, 
# "Massively parallel nonparametric regression, with an application
# to developmental brain mapping", to appear in JCGS.
# The original code for the analyses, by Huang, Chen and Reiss, was put in this self-contained
# form by Huo and Reiss. Please email phil.reiss@nyumc.org with any feedback.
# 
# Figures produced with this code will differ somewhat from those of the paper due to the 
# stochastic nature of the restricted likelihood ratio test. The exact original results 
# can be reproduced using vows version 0.1, available from the authors, by setting a seed
# prior to the RLRT.
 
# See the accompanying README file re installing the vows package
library(vows)

#### Input homotopic connectivity data
# Note: The following lines assume the data files reside in the working directory
age = read.table("age.txt")[ , 1]
age50 = age[age<=50]
load("homotopic-dat.RData")

#####################################################################

#### Fig. 3: Massively parallel RLRT
lisp = seq(-22,0,,100)
const.test = rlrt4d(d4, age50, loginvsp=lisp, nulldim=1)  
const.test$has.data = attributes(d4)$has.data
# pdf("fdr-slices.pdf", 6)
plot(const.test, d4, slices=5*1:11, disp="fdr", threshold=0.1, nrow=3, digit=3, titl="z=") 
# dev.off()

#####################################################################

#### Fig. 4: Massively parallel smoothing compared with mgcv
bsb = create.bspline.basis(range(age50), nbasis=15)
B = eval.basis(age50, bsb) 
P = getbasispenalty(bsb) 
yrnorm = rnorm(193)
df2root = function(df) uniroot(function(SP) df - sum(gam(yrnorm~B,paraPen=list(B=list(P)),sp=SP)$edf), c(0,exp(25)))
df2lsp = function(df) log(df2root(df)$root)
min.lsp = df2lsp(6)

ngrid = 100
lsp=seq(min.lsp, 22, length.out=ngrid)
qp2 = semipar4d(d4, ~sf(age50, k = 15), lsp, data=data.frame(age50 = age50),  store.reml = TRUE)

nvox = 200
set.seed(12867)
which.vox <- sample(sum(attr(d4,"has.data")),nvox)
data.inds = which(attr(d4,"has.data")==TRUE, arr.ind=TRUE)
logsp.mgcv <- edf.mgcv <- rep(NA,nvox)
fitted.mgcv = matrix(NA, nrow(B), nvox)

for (i in 1:nvox) {
    inds <- data.inds[which.vox[i], ]
    swmod = gam(d4[inds[1],inds[2],inds[3],]~B-1,paraPen=list(B=list(P)),min.sp=exp(min.lsp),method="REML")
    edf.mgcv[i] = sum(swmod$edf)
    logsp.mgcv[i]<-log(swmod$sp+exp(min.lsp))
    fitted.mgcv[ , i] = fitted(swmod)
}

locmax = which(edf.mgcv > qp2$pwdf[which.vox]+1)
vB = locmax[2]
vA = which(logsp.mgcv > qp2$pwlsp[which.vox]+1)

# pdf("mgcv-cf.pdf", 10)
par(mfrow=c(2,2))
plot(logsp.mgcv, qp2$pwlsp[which.vox], xlab="mgcv", ylab="Massively parallel", pch=20, main="(a) Log smoothing parameter")
abline(0,1)
text(logsp.mgcv[vA]+.4, qp2$pwlsp[which.vox][vA], "A")
text(logsp.mgcv[vB], qp2$pwlsp[which.vox][vB]-1, "B")

plot(edf.mgcv, qp2$pwdf[which.vox], xlab="mgcv", ylab="Massively parallel", pch=20,main="(b) Effective degrees of freedom")
abline(0,1)
text(edf.mgcv[vB], qp2$pwdf[which.vox][vB]+.25, "B")
text(edf.mgcv[vA], qp2$pwdf[which.vox][vA]+.25, "A")

matplot(sort(age50), cbind(d4[data.inds[which.vox[vB], 1], data.inds[which.vox[vB], 2], data.inds[which.vox[vB], 3], ], fitted.mgcv[,vB], cbind(1,B) %*% qp2$coef[, which.vox[vB]])[order(age50), ], type=c("p","l","l"), col=1, lty=c(1,1,2), pch=20, xlab="Age", ylab="Fitted value", main="(c) Fitted curves for voxel B")
legend("bottomright", lty=1:2, legend=c("mgcv","MP"))

plot(colnames(qp2$reml), qp2$reml[which.vox[vB], ], type="l", xlab=expression(paste("log(", lambda, ")")), ylab="REML criterion", main="(d) REML maxima for voxel B")
abline(v=logsp.mgcv[vB], lty=2)
text(logsp.mgcv[vB]+1, -259, "mgcv")
abline(v=qp2$pwlsp[which.vox][vB], lty=2)
text(qp2$pwlsp[which.vox][vB]-1, -259, "MP")
# dev.off()

#####################################################################

#### Fig. 5: Clustering of fitted curves
sv = screen.vox(qp2, d4, const.test$fdr<.1)
funk = funkmeans4d(sv, d4, ncomp = 6, center = 6)
funkpanel(funk, sv, d4, age50, deriv.legend=0) # ylim.legend=c(0, 1.4) 
